/**
 * DevExtreme (cjs/__internal/ui/context_menu/m_context_menu.js)
 * Version: 23.2.7 (build 24210-0111)
 * Build date: Sun Jul 28 2024
 *
 * Copyright (c) 2012 - 2024 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = void 0;
var _fx = _interopRequireDefault(require("../../../animation/fx"));
var _position = _interopRequireDefault(require("../../../animation/position"));
var _component_registrator = _interopRequireDefault(require("../../../core/component_registrator"));
var _devices = _interopRequireDefault(require("../../../core/devices"));
var _dom_adapter = _interopRequireDefault(require("../../../core/dom_adapter"));
var _element = require("../../../core/element");
var _guid = _interopRequireDefault(require("../../../core/guid"));
var _renderer = _interopRequireDefault(require("../../../core/renderer"));
var _common = require("../../../core/utils/common");
var _deferred = require("../../../core/utils/deferred");
var _dom = require("../../../core/utils/dom");
var _extend = require("../../../core/utils/extend");
var _iterator = require("../../../core/utils/iterator");
var _type = require("../../../core/utils/type");
var _window = require("../../../core/utils/window");
var _contextmenu = require("../../../events/contextmenu");
var _events_engine = _interopRequireDefault(require("../../../events/core/events_engine"));
var _hold = _interopRequireDefault(require("../../../events/hold"));
var _index = require("../../../events/utils/index");
var _ui = _interopRequireDefault(require("../../../ui/overlay/ui.overlay"));
var _m_menu_base = _interopRequireDefault(require("../../ui/context_menu/m_menu_base"));

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        default: obj
    }
}

function _inheritsLoose(subClass, superClass) {
    subClass.prototype = Object.create(superClass.prototype);
    subClass.prototype.constructor = subClass;
    _setPrototypeOf(subClass, superClass)
}

function _setPrototypeOf(o, p) {
    _setPrototypeOf = Object.setPrototypeOf ? Object.setPrototypeOf.bind() : function(o, p) {
        o.__proto__ = p;
        return o
    };
    return _setPrototypeOf(o, p)
}
const DX_MENU_CLASS = "dx-menu";
const DX_MENU_ITEM_CLASS = "".concat("dx-menu", "-item");
const DX_MENU_ITEM_EXPANDED_CLASS = "".concat(DX_MENU_ITEM_CLASS, "-expanded");
const DX_MENU_PHONE_CLASS = "dx-menu-phone-overlay";
const DX_MENU_ITEMS_CONTAINER_CLASS = "".concat("dx-menu", "-items-container");
const DX_MENU_ITEM_WRAPPER_CLASS = "".concat(DX_MENU_ITEM_CLASS, "-wrapper");
const DX_SUBMENU_CLASS = "dx-submenu";
const DX_CONTEXT_MENU_CLASS = "dx-context-menu";
const DX_HAS_CONTEXT_MENU_CLASS = "dx-has-context-menu";
const DX_STATE_DISABLED_CLASS = "dx-state-disabled";
const DX_STATE_FOCUSED_CLASS = "dx-state-focused";
const DX_STATE_HOVER_CLASS = "dx-state-hover";
const OVERLAY_CONTENT_CLASS = "dx-overlay-content";
const FOCUS_UP = "up";
const FOCUS_DOWN = "down";
const FOCUS_LEFT = "left";
const FOCUS_RIGHT = "right";
const FOCUS_FIRST = "first";
const FOCUS_LAST = "last";
const ACTIONS = ["onShowing", "onShown", "onSubmenuCreated", "onHiding", "onHidden", "onPositioning", "onLeftFirstItem", "onLeftLastItem", "onCloseRootSubmenu", "onExpandLastSubmenu"];
const LOCAL_SUBMENU_DIRECTIONS = ["up", "down", "first", "last"];
const DEFAULT_SHOW_EVENT = "dxcontextmenu";
const window = (0, _window.getWindow)();
let ContextMenu = function(_MenuBase) {
    _inheritsLoose(ContextMenu, _MenuBase);

    function ContextMenu() {
        return _MenuBase.apply(this, arguments) || this
    }
    var _proto = ContextMenu.prototype;
    _proto.getShowEvent = function(showEventOption) {
        var _a;
        if ((0, _type.isObject)(showEventOption)) {
            if (null !== showEventOption.name) {
                return null !== (_a = showEventOption.name) && void 0 !== _a ? _a : "dxcontextmenu"
            }
        } else {
            return showEventOption
        }
        return null
    };
    _proto.getShowDelay = function(showEventOption) {
        return (0, _type.isObject)(showEventOption) && showEventOption.delay
    };
    _proto._getDefaultOptions = function() {
        return (0, _extend.extend)(_MenuBase.prototype._getDefaultOptions.call(this), {
            showEvent: "dxcontextmenu",
            hideOnOutsideClick: true,
            position: {
                at: "top left",
                my: "top left"
            },
            onShowing: null,
            onShown: null,
            onSubmenuCreated: null,
            onHiding: null,
            onHidden: null,
            onPositioning: null,
            submenuDirection: "auto",
            visible: false,
            target: void 0,
            onLeftFirstItem: null,
            onLeftLastItem: null,
            onCloseRootSubmenu: null,
            onExpandLastSubmenu: null
        })
    };
    _proto._defaultOptionsRules = function() {
        return _MenuBase.prototype._defaultOptionsRules.call(this).concat([{
            device: () => !(0, _window.hasWindow)(),
            options: {
                animation: null
            }
        }])
    };
    _proto._setDeprecatedOptions = function() {
        _MenuBase.prototype._setDeprecatedOptions.call(this);
        (0, _extend.extend)(this._deprecatedOptions, {
            closeOnOutsideClick: {
                since: "22.2",
                alias: "hideOnOutsideClick"
            }
        })
    };
    _proto._initActions = function() {
        this._actions = {};
        (0, _iterator.each)(ACTIONS, (index, action) => {
            this._actions[action] = this._createActionByOption(action) || _common.noop
        })
    };
    _proto._setOptionsByReference = function() {
        _MenuBase.prototype._setOptionsByReference.call(this);
        (0, _extend.extend)(this._optionsByReference, {
            animation: true,
            selectedItem: true
        })
    };
    _proto._focusInHandler = function() {};
    _proto._itemContainer = function() {
        return this._overlay ? this._overlay.$content() : (0, _renderer.default)()
    };
    _proto._eventBindingTarget = function() {
        return this._itemContainer()
    };
    _proto.itemsContainer = function() {
        return this._overlay ? this._overlay.$content() : void 0
    };
    _proto._supportedKeys = function() {
        return (0, _extend.extend)(_MenuBase.prototype._supportedKeys.call(this), {
            space: () => {
                const $item = (0, _renderer.default)(this.option("focusedElement"));
                this.hide();
                if (!$item.length || !this._isSelectionEnabled()) {
                    return
                }
                this.selectItem($item[0])
            },
            escape: this.hide
        })
    };
    _proto._getActiveItem = function(last) {
        const $availableItems = this._getAvailableItems();
        const $focusedItem = $availableItems.filter(".".concat("dx-state-focused"));
        const $hoveredItem = $availableItems.filter(".".concat("dx-state-hover"));
        const $hoveredItemContainer = $hoveredItem.closest(".".concat(DX_MENU_ITEMS_CONTAINER_CLASS));
        if ($hoveredItemContainer.find(".".concat(DX_MENU_ITEM_CLASS)).index($focusedItem) >= 0) {
            return $focusedItem
        }
        if ($hoveredItem.length) {
            return $hoveredItem
        }
        return _MenuBase.prototype._getActiveItem.call(this)
    };
    _proto._moveFocus = function(location) {
        const $items = this._getItemsByLocation(location);
        const $oldTarget = this._getActiveItem(true);
        const $hoveredItem = this.itemsContainer().find(".".concat("dx-state-hover"));
        const $focusedItem = (0, _renderer.default)(this.option("focusedElement"));
        const $activeItemHighlighted = !!($focusedItem.length || $hoveredItem.length);
        let $newTarget;
        switch (location) {
            case "up":
                $newTarget = $activeItemHighlighted ? this._prevItem($items) : $oldTarget;
                this._setFocusedElement($newTarget);
                if ($oldTarget.is($items.first())) {
                    this._actions.onLeftFirstItem($oldTarget)
                }
                break;
            case "down":
                $newTarget = $activeItemHighlighted ? this._nextItem($items) : $oldTarget;
                this._setFocusedElement($newTarget);
                if ($oldTarget.is($items.last())) {
                    this._actions.onLeftLastItem($oldTarget)
                }
                break;
            case "right":
                $newTarget = this.option("rtlEnabled") ? this._hideSubmenuHandler() : this._expandSubmenuHandler($items, location);
                this._setFocusedElement($newTarget);
                break;
            case "left":
                $newTarget = this.option("rtlEnabled") ? this._expandSubmenuHandler($items, location) : this._hideSubmenuHandler();
                this._setFocusedElement($newTarget);
                break;
            case "first":
                $newTarget = $items.first();
                this._setFocusedElement($newTarget);
                break;
            case "last":
                $newTarget = $items.last();
                this._setFocusedElement($newTarget);
                break;
            default:
                return _MenuBase.prototype._moveFocus.call(this, location)
        }
    };
    _proto._setFocusedElement = function($element) {
        if ($element && 0 !== $element.length) {
            this.option("focusedElement", (0, _element.getPublicElement)($element))
        }
    };
    _proto._getItemsByLocation = function(location) {
        const $activeItem = this._getActiveItem(true);
        let $items;
        if (LOCAL_SUBMENU_DIRECTIONS.includes(location)) {
            $items = $activeItem.closest(".".concat(DX_MENU_ITEMS_CONTAINER_CLASS)).children().children()
        }
        $items = this._getAvailableItems($items);
        return $items
    };
    _proto._getAriaTarget = function() {
        return this.$element()
    };
    _proto._refreshActiveDescendant = function() {
        if ((0, _type.isDefined)(this._overlay)) {
            const $target = this._overlay.$content();
            _MenuBase.prototype._refreshActiveDescendant.call(this, $target)
        }
    };
    _proto._hideSubmenuHandler = function() {
        const $curItem = this._getActiveItem(true);
        const $parentItem = $curItem.parents(".".concat(DX_MENU_ITEM_EXPANDED_CLASS)).first();
        if ($parentItem.length) {
            this._hideSubmenusOnSameLevel($parentItem);
            this._hideSubmenu($curItem.closest(".".concat("dx-submenu")));
            return $parentItem
        }
        this._actions.onCloseRootSubmenu($curItem)
    };
    _proto._expandSubmenuHandler = function($items, location) {
        const $curItem = this._getActiveItem(true);
        const itemData = this._getItemData($curItem);
        const node = this._dataAdapter.getNodeByItem(itemData);
        const isItemHasSubmenu = this._hasSubmenu(node);
        const $submenu = $curItem.children(".".concat("dx-submenu"));
        if (isItemHasSubmenu && !$curItem.hasClass("dx-state-disabled")) {
            if (!$submenu.length || "hidden" === $submenu.css("visibility")) {
                this._showSubmenu($curItem)
            }
            return this._nextItem(this._getItemsByLocation(location))
        }
        this._actions.onExpandLastSubmenu($curItem);
        return
    };
    _proto._clean = function() {
        if (this._overlay) {
            this._overlay.$element().remove();
            this._overlay = null
        }
        this._detachShowContextMenuEvents(this._getTarget());
        _MenuBase.prototype._clean.call(this)
    };
    _proto._initMarkup = function() {
        this.$element().addClass("dx-has-context-menu");
        _MenuBase.prototype._initMarkup.call(this)
    };
    _proto._render = function() {
        _MenuBase.prototype._render.call(this);
        this._renderVisibility(this.option("visible"));
        this._addWidgetClass()
    };
    _proto._renderContentImpl = function() {
        this._detachShowContextMenuEvents(this._getTarget());
        this._attachShowContextMenuEvents()
    };
    _proto._attachKeyboardEvents = function() {
        !this._keyboardListenerId && this._focusTarget().length && _MenuBase.prototype._attachKeyboardEvents.call(this)
    };
    _proto._renderContextMenuOverlay = function() {
        if (this._overlay) {
            return
        }
        const overlayOptions = this._getOverlayOptions();
        this._overlay = this._createComponent((0, _renderer.default)("<div>").appendTo(this._$element), _ui.default, overlayOptions);
        const $overlayContent = this._overlay.$content();
        $overlayContent.addClass("dx-context-menu");
        this._addCustomCssClass($overlayContent);
        this._addPlatformDependentClass($overlayContent);
        this._attachContextMenuEvent()
    };
    _proto.preventShowingDefaultContextMenuAboveOverlay = function() {
        const $itemContainer = this._itemContainer();
        const eventName = (0, _index.addNamespace)(_contextmenu.name, this.NAME);
        _events_engine.default.off($itemContainer, eventName, ".".concat("dx-submenu"));
        _events_engine.default.on($itemContainer, eventName, ".".concat("dx-submenu"), e => {
            e.stopPropagation();
            e.preventDefault();
            _events_engine.default.off($itemContainer, eventName, ".".concat("dx-submenu"))
        })
    };
    _proto._itemContextMenuHandler = function(e) {
        _MenuBase.prototype._itemContextMenuHandler.call(this, e);
        e.stopPropagation()
    };
    _proto._addPlatformDependentClass = function($element) {
        if (_devices.default.current().phone) {
            $element.addClass(DX_MENU_PHONE_CLASS)
        }
    };
    _proto._detachShowContextMenuEvents = function(target) {
        const showEvent = this.getShowEvent(this.option("showEvent"));
        if (!showEvent) {
            return
        }
        const eventName = (0, _index.addNamespace)(showEvent, this.NAME);
        if (this._showContextMenuEventHandler) {
            _events_engine.default.off(_dom_adapter.default.getDocument(), eventName, target, this._showContextMenuEventHandler)
        } else {
            _events_engine.default.off((0, _renderer.default)(target), eventName)
        }
    };
    _proto._attachShowContextMenuEvents = function() {
        const target = this._getTarget();
        const showEvent = this.getShowEvent(this.option("showEvent"));
        if (!showEvent) {
            return
        }
        const eventName = (0, _index.addNamespace)(showEvent, this.NAME);
        let contextMenuAction = this._createAction(e => {
            const delay = this.getShowDelay(this.option("showEvent"));
            if (delay) {
                setTimeout(() => this._show(e.event), delay)
            } else {
                this._show(e.event)
            }
        }, {
            validatingTargetName: "target"
        });
        const handler = e => contextMenuAction({
            event: e,
            target: (0, _renderer.default)(e.currentTarget)
        });
        contextMenuAction = this._createAction(contextMenuAction);
        if ((0, _type.isRenderer)(target) || target.nodeType || (0, _type.isWindow)(target)) {
            this._showContextMenuEventHandler = void 0;
            _events_engine.default.on(target, eventName, handler)
        } else {
            this._showContextMenuEventHandler = handler;
            _events_engine.default.on(_dom_adapter.default.getDocument(), eventName, target, this._showContextMenuEventHandler)
        }
    };
    _proto._hoverEndHandler = function(e) {
        _MenuBase.prototype._hoverEndHandler.call(this, e);
        e.stopPropagation()
    };
    _proto._renderDimensions = function() {};
    _proto._renderContainer = function($wrapper, submenuContainer) {
        const $holder = submenuContainer || this._itemContainer();
        $wrapper = (0, _renderer.default)("<div>");
        $wrapper.appendTo($holder).addClass("dx-submenu").css("visibility", submenuContainer ? "hidden" : "visible");
        if (!$wrapper.parent().hasClass("dx-overlay-content")) {
            this._addCustomCssClass($wrapper)
        }
        const $itemsContainer = _MenuBase.prototype._renderContainer.call(this, $wrapper);
        if (submenuContainer) {
            return $itemsContainer
        }
        if (this.option("width")) {
            return $itemsContainer.css("minWidth", this.option("width"))
        }
        if (this.option("height")) {
            return $itemsContainer.css("minHeight", this.option("height"))
        }
        return $itemsContainer
    };
    _proto._renderSubmenuItems = function(node, $itemFrame) {
        this._renderItems(this._getChildNodes(node), $itemFrame);
        this._actions.onSubmenuCreated({
            itemElement: (0, _element.getPublicElement)($itemFrame),
            itemData: node.internalFields.item,
            submenuElement: (0, _element.getPublicElement)($itemFrame.children(".".concat("dx-submenu")))
        })
    };
    _proto._getOverlayOptions = function() {
        const position = this.option("position");
        const overlayOptions = {
            focusStateEnabled: this.option("focusStateEnabled"),
            animation: this.option("animation"),
            innerOverlay: true,
            hideOnOutsideClick: e => this._hideOnOutsideClickHandler(e),
            propagateOutsideClick: true,
            hideOnParentScroll: true,
            deferRendering: false,
            position: {
                at: position.at,
                my: position.my,
                of: this._getTarget(),
                collision: "flipfit"
            },
            shading: false,
            showTitle: false,
            height: "auto",
            width: "auto",
            onShown: this._overlayShownActionHandler.bind(this),
            onHiding: this._overlayHidingActionHandler.bind(this),
            onHidden: this._overlayHiddenActionHandler.bind(this),
            visualContainer: window
        };
        return overlayOptions
    };
    _proto._overlayShownActionHandler = function(arg) {
        this._actions.onShown(arg)
    };
    _proto._overlayHidingActionHandler = function(arg) {
        this._actions.onHiding(arg);
        if (!arg.cancel) {
            this._hideAllShownSubmenus();
            this._setOptionWithoutOptionChange("visible", false)
        }
    };
    _proto._overlayHiddenActionHandler = function(arg) {
        this._actions.onHidden(arg)
    };
    _proto._shouldHideOnOutsideClick = function(e) {
        const {
            closeOnOutsideClick: closeOnOutsideClick,
            hideOnOutsideClick: hideOnOutsideClick
        } = this.option();
        if ((0, _type.isFunction)(hideOnOutsideClick)) {
            return hideOnOutsideClick(e)
        }
        if ((0, _type.isFunction)(closeOnOutsideClick)) {
            return closeOnOutsideClick(e)
        }
        return hideOnOutsideClick || closeOnOutsideClick
    };
    _proto._hideOnOutsideClickHandler = function(e) {
        if (!this._shouldHideOnOutsideClick(e)) {
            return false
        }
        if (_dom_adapter.default.isDocument(e.target)) {
            return true
        }
        const $activeItemContainer = this._getActiveItemsContainer(e.target);
        const $itemContainers = this._getItemsContainers();
        const $clickedItem = this._searchActiveItem(e.target);
        const $rootItem = this.$element().parents(".".concat(DX_MENU_ITEM_CLASS));
        const isRootItemClicked = $clickedItem[0] === $rootItem[0] && $clickedItem.length && $rootItem.length;
        const isInnerOverlayClicked = this._isIncludeOverlay($activeItemContainer, $itemContainers) && $clickedItem.length;
        if (isInnerOverlayClicked || isRootItemClicked) {
            if ("onClick" === this._getShowSubmenuMode()) {
                this._hideAllShownChildSubmenus($clickedItem)
            }
            return false
        }
        return true
    };
    _proto._getActiveItemsContainer = function(target) {
        return (0, _renderer.default)(target).closest(".".concat(DX_MENU_ITEMS_CONTAINER_CLASS))
    };
    _proto._getItemsContainers = function() {
        return this._overlay.$content().find(".".concat(DX_MENU_ITEMS_CONTAINER_CLASS))
    };
    _proto._searchActiveItem = function(target) {
        return (0, _renderer.default)(target).closest(".".concat(DX_MENU_ITEM_CLASS)).eq(0)
    };
    _proto._isIncludeOverlay = function($activeOverlay, $allOverlays) {
        let isSame = false;
        (0, _iterator.each)($allOverlays, (index, $overlay) => {
            if ($activeOverlay.is($overlay) && !isSame) {
                isSame = true
            }
        });
        return isSame
    };
    _proto._hideAllShownChildSubmenus = function($clickedItem) {
        const $submenuElements = $clickedItem.find(".".concat("dx-submenu"));
        const shownSubmenus = (0, _extend.extend)([], this._shownSubmenus);
        if ($submenuElements.length > 0) {
            (0, _iterator.each)(shownSubmenus, (index, $submenu) => {
                const $context = this._searchActiveItem($submenu.context).parent();
                if ($context.parent().is($clickedItem.parent().parent()) && !$context.is($clickedItem.parent())) {
                    this._hideSubmenu($submenu)
                }
            })
        }
    };
    _proto._showSubmenu = function($item) {
        const node = this._dataAdapter.getNodeByItem(this._getItemData($item));
        this._hideSubmenusOnSameLevel($item);
        if (!this._hasSubmenu(node)) {
            return
        }
        const $submenu = $item.children(".".concat("dx-submenu"));
        const isSubmenuRendered = $submenu.length;
        _MenuBase.prototype._showSubmenu.call(this, $item);
        if (!isSubmenuRendered) {
            this._renderSubmenuItems(node, $item)
        }
        if (!this._isSubmenuVisible($submenu)) {
            this._drawSubmenu($item)
        }
    };
    _proto._hideSubmenusOnSameLevel = function($item) {
        const $expandedItems = $item.parent(".".concat(DX_MENU_ITEM_WRAPPER_CLASS)).siblings().find(".".concat(DX_MENU_ITEM_EXPANDED_CLASS));
        if ($expandedItems.length) {
            $expandedItems.removeClass(DX_MENU_ITEM_EXPANDED_CLASS);
            this._hideSubmenu($expandedItems.find(".".concat("dx-submenu")))
        }
    };
    _proto._hideSubmenuGroup = function($submenu) {
        if (this._isSubmenuVisible($submenu)) {
            this._hideSubmenuCore($submenu)
        }
    };
    _proto._isSubmenuVisible = function($submenu) {
        return "visible" === $submenu.css("visibility")
    };
    _proto._drawSubmenu = function($itemElement) {
        const animation = this.option("animation") ? this.option("animation").show : {};
        const $submenu = $itemElement.children(".".concat("dx-submenu"));
        const submenuPosition = this._getSubmenuPosition($itemElement);
        if (this._overlay && this._overlay.option("visible")) {
            if (!(0, _type.isDefined)(this._shownSubmenus)) {
                this._shownSubmenus = []
            }
            if (!this._shownSubmenus.includes($submenu)) {
                this._shownSubmenus.push($submenu)
            }
            if (animation) {
                _fx.default.stop($submenu)
            }
            _position.default.setup($submenu, submenuPosition);
            if (animation) {
                if ((0, _type.isPlainObject)(animation.to)) {
                    animation.to.position = submenuPosition
                }
                this._animate($submenu, animation)
            }
            $submenu.css("visibility", "visible")
        }
    };
    _proto._animate = function($container, options) {
        _fx.default.animate($container, options)
    };
    _proto._getSubmenuPosition = function($rootItem) {
        const submenuDirection = this.option("submenuDirection").toLowerCase();
        const $rootItemWrapper = $rootItem.parent(".".concat(DX_MENU_ITEM_WRAPPER_CLASS));
        const position = {
            collision: "flip",
            of: $rootItemWrapper,
            offset: {
                h: 0,
                v: -1
            }
        };
        switch (submenuDirection) {
            case "left":
                position.at = "left top";
                position.my = "right top";
                break;
            case "right":
                position.at = "right top";
                position.my = "left top";
                break;
            default:
                if (this.option("rtlEnabled")) {
                    position.at = "left top";
                    position.my = "right top"
                } else {
                    position.at = "right top";
                    position.my = "left top"
                }
        }
        return position
    };
    _proto._updateSubmenuVisibilityOnClick = function(actionArgs) {
        if (!actionArgs.args.length) {
            return
        }
        const {
            itemData: itemData
        } = actionArgs.args[0];
        const node = this._dataAdapter.getNodeByItem(itemData);
        if (!node) {
            return
        }
        const $itemElement = (0, _renderer.default)(actionArgs.args[0].itemElement);
        let $submenu = $itemElement.find(".".concat("dx-submenu"));
        const shouldRenderSubmenu = this._hasSubmenu(node) && !$submenu.length;
        if (shouldRenderSubmenu) {
            this._renderSubmenuItems(node, $itemElement);
            $submenu = $itemElement.find(".".concat("dx-submenu"))
        }
        if ($itemElement.context === $submenu.context && "visible" === $submenu.css("visibility")) {
            return
        }
        this._updateSelectedItemOnClick(actionArgs);
        const notCloseMenuOnItemClick = itemData && false === itemData.closeMenuOnClick;
        if (!itemData || itemData.disabled || notCloseMenuOnItemClick) {
            return
        }
        if (0 === $submenu.length) {
            const $prevSubmenu = (0, _renderer.default)($itemElement.parents(".".concat("dx-submenu"))[0]);
            this._hideSubmenu($prevSubmenu);
            if (!actionArgs.canceled && this._overlay && this._overlay.option("visible")) {
                this.option("visible", false)
            }
        } else {
            if (this._shownSubmenus && this._shownSubmenus.length > 0) {
                if (this._shownSubmenus[0].is($submenu)) {
                    this._hideSubmenu($submenu)
                }
            }
            this._showSubmenu($itemElement)
        }
    };
    _proto._hideSubmenu = function($curSubmenu) {
        const shownSubmenus = (0, _extend.extend)([], this._shownSubmenus);
        (0, _iterator.each)(shownSubmenus, (index, $submenu) => {
            if ($curSubmenu.is($submenu) || (0, _dom.contains)($curSubmenu[0], $submenu[0])) {
                $submenu.parent().removeClass(DX_MENU_ITEM_EXPANDED_CLASS);
                this._hideSubmenuCore($submenu)
            }
        })
    };
    _proto._hideSubmenuCore = function($submenu) {
        const index = this._shownSubmenus.indexOf($submenu);
        const animation = this.option("animation") ? this.option("animation").hide : null;
        if (index >= 0) {
            this._shownSubmenus.splice(index, 1)
        }
        this._stopAnimate($submenu);
        animation && this._animate($submenu, animation);
        $submenu.css("visibility", "hidden")
    };
    _proto._stopAnimate = function($container) {
        _fx.default.stop($container, true)
    };
    _proto._hideAllShownSubmenus = function() {
        const shownSubmenus = (0, _extend.extend)([], this._shownSubmenus);
        const $expandedItems = this._overlay.$content().find(".".concat(DX_MENU_ITEM_EXPANDED_CLASS));
        $expandedItems.removeClass(DX_MENU_ITEM_EXPANDED_CLASS);
        (0, _iterator.each)(shownSubmenus, (_, $submenu) => {
            this._hideSubmenu($submenu)
        })
    };
    _proto._visibilityChanged = function(visible) {
        if (visible) {
            this._renderContentImpl()
        }
    };
    _proto._optionChanged = function(args) {
        if (ACTIONS.includes(args.name)) {
            this._initActions();
            return
        }
        switch (args.name) {
            case "visible":
                this._renderVisibility(args.value);
                break;
            case "showEvent":
            case "position":
            case "submenuDirection":
                this._invalidate();
                break;
            case "target":
                args.previousValue && this._detachShowContextMenuEvents(args.previousValue);
                this._invalidate();
                break;
            case "closeOnOutsideClick":
            case "hideOnOutsideClick":
                break;
            default:
                _MenuBase.prototype._optionChanged.call(this, args)
        }
    };
    _proto._renderVisibility = function(showing) {
        return showing ? this._show() : this._hide()
    };
    _proto._toggleVisibility = function() {};
    _proto._show = function(event) {
        var _a;
        const args = {
            jQEvent: event
        };
        let promise = (0, _deferred.Deferred)().reject().promise();
        this._actions.onShowing(args);
        if (args.cancel) {
            return promise
        }
        const position = this._positionContextMenu(event);
        if (position) {
            if (!this._overlay) {
                this._renderContextMenuOverlay();
                this._overlay.$content().addClass(this._widgetClass());
                this._renderFocusState();
                this._attachHoverEvents();
                this._attachClickEvent();
                this._renderItems(this._dataAdapter.getRootNodes())
            }
            this._setOptionWithoutOptionChange("visible", true);
            this._overlay.option("position", position);
            promise = this._overlay.show();
            event && event.stopPropagation();
            this._setAriaAttributes();
            if ((null === (_a = null === event || void 0 === event ? void 0 : event.originalEvent) || void 0 === _a ? void 0 : _a.type) === _hold.default.name) {
                this.preventShowingDefaultContextMenuAboveOverlay()
            }
        }
        return promise
    };
    _proto._setAriaAttributes = function() {
        this._overlayContentId = "dx-".concat(new _guid.default);
        this.setAria("owns", this._overlayContentId);
        this.setAria({
            id: this._overlayContentId,
            role: "menu"
        }, this._overlay.$content())
    };
    _proto._cleanAriaAttributes = function() {
        this._overlay && this.setAria("id", null, this._overlay.$content());
        this.setAria("owns", void 0)
    };
    _proto._getTarget = function() {
        return this.option("target") || this.option("position").of || (0, _renderer.default)(_dom_adapter.default.getDocument())
    };
    _proto._getContextMenuPosition = function() {
        return (0, _extend.extend)({}, this.option("position"), {
            of: this._getTarget()
        })
    };
    _proto._positionContextMenu = function(jQEvent) {
        let position = this._getContextMenuPosition();
        const isInitialPosition = this._isInitialOptionValue("position");
        const positioningAction = this._createActionByOption("onPositioning");
        if (jQEvent && jQEvent.preventDefault && isInitialPosition) {
            position.of = jQEvent
        }
        const actionArgs = {
            position: position,
            event: jQEvent
        };
        positioningAction(actionArgs);
        if (actionArgs.cancel) {
            position = null
        } else if (actionArgs.event) {
            actionArgs.event.cancel = true;
            jQEvent.preventDefault()
        }
        return position
    };
    _proto._refresh = function() {
        if (!(0, _window.hasWindow)()) {
            _MenuBase.prototype._refresh.call(this)
        } else if (this._overlay) {
            const lastPosition = this._overlay.option("position");
            _MenuBase.prototype._refresh.call(this);
            this._overlay && this._overlay.option("position", lastPosition)
        } else {
            _MenuBase.prototype._refresh.call(this)
        }
    };
    _proto._hide = function() {
        let promise;
        if (this._overlay) {
            promise = this._overlay.hide();
            this._setOptionWithoutOptionChange("visible", false)
        }
        this._cleanAriaAttributes();
        this.option("focusedElement", null);
        return promise || (0, _deferred.Deferred)().reject().promise()
    };
    _proto.toggle = function(showing) {
        const visible = this.option("visible");
        showing = void 0 === showing ? !visible : showing;
        return this._renderVisibility(showing)
    };
    _proto.show = function() {
        return this.toggle(true)
    };
    _proto.hide = function() {
        return this.toggle(false)
    };
    return ContextMenu
}(_m_menu_base.default);
(0, _component_registrator.default)("dxContextMenu", ContextMenu);
var _default = ContextMenu;
exports.default = _default;
